#include <stdio.h>
#include <string.h>
#include "xio.h"
#include "xparameters.h"
#include "xgpio_l.h"
#include "TestApp_Peripheral.h"
#include "uart.h"
#include "ddr.h"
#include "sleep.h"

extern int fVerbose;

#define MEMORY_DATA (*(unsigned*)MEMORY_ADDRESS)
unsigned MEMORY_ADDRESS;


int mtstNew()
{
	unsigned x = 0;
	int error = 0;
	unsigned test_resolution = 0x100; // address increment value 
	unsigned ddr_prog_addr = XPAR_DDR_SDRAM_16MX32_MEM0_BASEADDR;
	unsigned ddr_prog_data = XPAR_DDR_SDRAM_16MX32_MEM0_BASEADDR;
	unsigned char chGet = 0;
	unsigned i =0;

	// program some values
	if (fVerbose) {
		print("\r\nWriting Addresses...\r\n");
	}
	
	//write the first value
	ddr_prog_addr=0x00000000;
	ddr_prog_data=0xFFFFFFFE;
	MEMORY_ADDRESS = XPAR_DDR_SDRAM_16MX32_MEM0_BASEADDR + ddr_prog_addr;
	MEMORY_DATA = ddr_prog_data;

	if (fVerbose) xil_printf("\rWriting address %x, data %x", ddr_prog_addr, ddr_prog_data);

	for (i=0;i<32;i++) { //write the other values
			if (i<24) ddr_prog_addr=(0xFFFFFF ^ (0x1 << i))<<2;
		   else ddr_prog_addr = ddr_prog_addr + 0x00000004; 
			
			//there are more data lines than address lines
			//so, for the remaininng walking 0 data lines, different addresses will be used
			
			ddr_prog_data=(0xFFFFFFFF ^ (0x2 << i)); //the first 0 was already written 
			if (fVerbose) xil_printf("\rWriting address %x, data %x", ddr_prog_addr, ddr_prog_data);
			MEMORY_ADDRESS = XPAR_DDR_SDRAM_16MX32_MEM0_BASEADDR + ddr_prog_addr;
			MEMORY_DATA = ddr_prog_data;
		}

	// read some values
	if (fVerbose) {
		print("\r\nReading Addresses...\r\n");
	}
	
	//read the first value
	ddr_prog_addr=0x00000000;
	ddr_prog_data=0xFFFFFFFE;
	MEMORY_ADDRESS = XPAR_DDR_SDRAM_16MX32_MEM0_BASEADDR + ddr_prog_addr;
	x = MEMORY_DATA;
	if (fVerbose) xil_printf("\rReading address %x, data %x", ddr_prog_addr, ddr_prog_data);
		if (x != ddr_prog_data) { 
			if (fVerbose) {
				print("\r\n\033[5mMem error\033[0m\r\n");
				print("At Address: 0x");
				putnum(ddr_prog_addr);
				print("\r\nExpected: 0x");
				putnum(ddr_prog_data);
				print("   Got: 0x");
				putnum(x);
				print("\r\n");
			}
			error++;
		}
	
	for (i=0;i<32;i++)//write the other values
		{
			if (i<24) ddr_prog_addr=(0xFFFFFF ^ (0x1 << i))<<2;
		   else ddr_prog_addr = ddr_prog_addr + 0x00000004; 
			
			//there are more data lines than address lines
			//so, for the remaininng walking 0 data lines, different adresses will be used
			
			ddr_prog_data=(0xFFFFFFFF ^ (0x2 << i)); //the first 0 was already written 
			if (fVerbose) xil_printf("\rReading address %x, data %x", ddr_prog_addr, ddr_prog_data);
			MEMORY_ADDRESS = XPAR_DDR_SDRAM_16MX32_MEM0_BASEADDR + ddr_prog_addr;
			x = MEMORY_DATA;
			if (x != ddr_prog_data) { 
				if (fVerbose) {
					print("\r\n\033[5mMem error\033[0m\r\n");
					print("At Address: 0x");
					putnum(ddr_prog_addr);
					print("\r\nExpected: 0x");
					putnum(ddr_prog_data);
					print("   Got: 0x");
					putnum(x);
					print("\r\n");
				}
				error++;
			}
		
		}

	if (error == 0) {
		if (fVerbose) {
			print("\r\nDDR Test Passed!\r\n");
		}
		return 1;
	}
	else {
		return 0;
	}
}

/////////////////////////////////////////////
//  DDR SDRAM Check                        //
/////////////////////////////////////////////
int ddr_check()
{
	return mtstNew();
}

/////////////////////////////////////////////
//  Flash Check                            //
/////////////////////////////////////////////
int flash_check()
{
	unsigned x = 0;
	int error = 0;
	unsigned i = 0;
	unsigned test_resolution = 0x100; // address increment value 
	unsigned flash_prog_addr = XPAR_FLASH_4MX32_MEM0_BASEADDR;
	unsigned flash_prog_data = XPAR_FLASH_4MX32_MEM0_BASEADDR;
	unsigned char chGet = 0;
	static int Flash_Block_List[]={0, 70, 71, 102, 118, 126, 130, 132, 133, 134, -1}; 
	//last one is -1 to mark string end

	/* put flash in Autoselect */
	Flash_Autoselect(XPAR_FLASH_4MX32_MEM0_BASEADDR);

	/* check to see if we are talking to ST Flash */
	MEMORY_ADDRESS = XPAR_FLASH_4MX32_MEM0_BASEADDR; 		//set the address 
	x = MEMORY_DATA;                                 		//read the data
    if(x != 0x00200020) {
		if (fVerbose) {
			xil_printf("Invalid Manufacturer ID: %x\r\n", x);
		}
		return 0;
	}
	MEMORY_ADDRESS = XPAR_FLASH_4MX32_MEM0_BASEADDR + 4; 	//set the address 
	x = MEMORY_DATA;                                 		//read the data
    if(x != 0x22DF22DF) {
		if (fVerbose) {
			xil_printf("\r\nInvalid device ID: %x\r\n", x);
			error++;
		}
		return 0;
	}
	if (fVerbose) {
		print("\r\nST M29W640DB Identified.\r\n");
	}

		
	if (fVerbose) {
		print("Erasing Flash");
	}

	/* erase the flash */
	i= Flash_Block_Erase(XPAR_FLASH_4MX32_MEM0_BASEADDR, &Flash_Block_List);

	if (fVerbose)
	{
		if (i)
		{
		xil_printf("\r\nErase error\r\n");
		error++;
		}
		else xil_printf("\r\nErase succesfull\r\n");		
	}
	
	// write the values
		if (fVerbose) {
		print("Writing addresses ...\r\n");
	}
	
	//walk 0 on the line
	//write address 0
	flash_prog_addr=0x00000000;
	flash_prog_data=0xFFFFFFFE;
	
	if (fVerbose) xil_printf("\rWriting address %x, data %x", flash_prog_addr, flash_prog_data);
	
	x = Flash_Write(XPAR_FLASH_4MX32_MEM0_BASEADDR, flash_prog_addr, flash_prog_data);
	for (i=0;i<32;i++) { //write the other values

			if (i<22) flash_prog_addr=(0x3FFFFF ^ (0x1 << i))<<2;
		   else flash_prog_addr = flash_prog_addr + 0x00000004; 
			//there are more data lines than address lines
			//so, for the remaininng walking 0 data lines, different adresses will be used
			
			flash_prog_data=(0xFFFFFFFF ^ (0x2 << i)); //the first 0 was already written 

			if (fVerbose) xil_printf("\rWriting address %x, data %x", flash_prog_addr, flash_prog_data);
		
			x = Flash_Write(XPAR_FLASH_4MX32_MEM0_BASEADDR, flash_prog_addr, flash_prog_data);
		}
	
	//reset the flash in read mode
	Flash_Reset_Read(XPAR_FLASH_4MX32_MEM0_BASEADDR);

	// read back the adresses
	if (fVerbose) {
		print("\r\nReading addresses ...\r\n");
	}

	flash_prog_addr=0x00000000;
	flash_prog_data=0xFFFFFFFE;
	
	if (fVerbose) xil_printf("\rReading address %x, data %x", flash_prog_addr, flash_prog_data);
	
	MEMORY_ADDRESS = XPAR_FLASH_4MX32_MEM0_BASEADDR + flash_prog_addr;
	x = MEMORY_DATA;

	if (x != flash_prog_data) {
			if (fVerbose) {
				print("\r\n\033[5mMem error\033[0m\r\n");
				print("At Address: 0x");
				putnum(flash_prog_addr);
				print("\r\nExpected: 0x");
				putnum(flash_prog_data);
				print("   Got: 0x");
				putnum(x);
				print("\r\n");
			}
			error ++;
	}
	
	for (i=0;i<32;i++) {//read the other values

			if (i<22) flash_prog_addr=(0x3FFFFF ^ (0x1 << i))<<2;
		   else flash_prog_addr = flash_prog_addr + 0x00000004;

			flash_prog_data=(0xFFFFFFFF ^ (0x2 << i));

			if (fVerbose) xil_printf("\rReading address %x, data %x", flash_prog_addr, flash_prog_data);

			MEMORY_ADDRESS = XPAR_FLASH_4MX32_MEM0_BASEADDR + flash_prog_addr;
			x = MEMORY_DATA;
			if (x != flash_prog_data) { 
				if (fVerbose) {
					print("\r\n\033[5mMem error\033[0m\r\n");
					print("At Address: 0x");
					putnum(flash_prog_addr);
					print("\r\nExpected: 0x");
					putnum(flash_prog_data);
					print("   Got: 0x");
					putnum(x);
					print("\r\n");
				}
				error ++;
			}
		}

	if (error == 0) {
		if (fVerbose) {
			print("\r\nFlash Test Passed!\r\n");
		}
		return 1;
	}
	else {
		return 0;
	}
}

