/************************************************************************/
/*                                                                      */
/*  main.c	--  contains main entrance to BIST and main polling loop	*/
/*                                                                      */
/************************************************************************/
/*	Author:		Joshua Pederson											*/
/*	Copyright 2006: Digilent, Inc.										*/
/************************************************************************/
/*  Module Description: 												*/
/*                                                                      */
/*                                                                      */
/************************************************************************/
/*  Revision History:													*/
/*                                                                      */
/*	08/1/2006(JPederson): created										*/
/*                                                                      */
/************************************************************************/

/* ------------------------------------------------------------ */
/*					Include File Definitions					*/
/* ------------------------------------------------------------ */

#include "xio.h"
#include "xparameters.h"
#include "xgpio_l.h"
#include "TestApp_Peripheral.h"
#include "uart.h"
#include "xrom_lcd.h"
#include "ddr.h"
#include "ping.h"
#include "sleep.h"

/* ------------------------------------------------------------ */
/*					Local Type Definitions						*/
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*					 Global Variables							*/
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*					Local Variables								*/
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*					Forward Declarations						*/
/* ------------------------------------------------------------ */

void DoTestLoop();
int FDoFx2OpenTest();

int fVerbose;


void DoShowMenu() {

	print("----------------------------------------------------------\r\n");
   print("Welcome to NU Horizons FX12\r\n");
   print("----------------------------------------------------------\r\n");
	print("\tTest Options:\r\n");
	print("\t 1.  Test DDR Memory\r\n");
	print("\t 2.  Test Flash Memory\r\n");
	print("\t 3.  Test Ethernet\r\n");
	print(">");

}


/* ------------------------------------------------------------ */
/*					Procedure Definitions						*/
/* ------------------------------------------------------------ */
/***	Main
**
**  Synopsis
**		Main()
**
**  Input:
**		none
**
**  Output:
**		returns 0
**
**  Errors:
**		none
**
**  Description:
**		This routine is the entrance to the BIST and calls the 
**		main polling loop for buttons, LEDs, and received commands
**		via the RS232 port.
*/

int
main(){

	//unsigned char chRead;
	
	Xuint8 chRead;
	unsigned int buttons;

	fVerbose = 1;

	//Initialize Components
	uartInit(XPAR_OPB_UARTLITE_0_BASEADDR);
	clearScreen();

	DoShowMenu();
	
	XromLCDOn();
	XromLCDInit();
	XromLCDPrint2Strings ("Welcome to", "NU Horizons FX12");

	XGpio_mSetDataDirection(XPAR_CHAR_LCD_BASEADDR, 1, 0xffffffff);   /* Set as outputs */
	buttons = XIo_In32(XPAR_CHAR_LCD_BASEADDR);

	XGpio_mSetDataDirection(XPAR_LEDS_BASEADDR, 1, 0x00000000);
	while(1) {


		//Poll Buttons
	   if(	XIo_In32(XPAR_CHAR_LCD_BASEADDR)!=buttons) {


			if ((XIo_In32(XPAR_CHAR_LCD_BASEADDR)^buttons)& 0x8) {

				if(XIo_In32(XPAR_CHAR_LCD_BASEADDR) & 0x8) {
					XGpio_mSetDataReg(XPAR_LEDS_BASEADDR, 1, 0x03);
					buttons = buttons + 8;
				}
				else {
					XGpio_mSetDataReg(XPAR_LEDS_BASEADDR, 1, 0x00);
					buttons = buttons - 8;
				}
			}

			if ((XIo_In32(XPAR_CHAR_LCD_BASEADDR)^buttons)& 0x4) {

				if(XIo_In32(XPAR_CHAR_LCD_BASEADDR) & 0x4) {
					XGpio_mSetDataReg(XPAR_LEDS_BASEADDR, 1, 0x0c);
					buttons = buttons + 4;
				}
				else {
					XGpio_mSetDataReg(XPAR_LEDS_BASEADDR, 1, 0x00);
					buttons = buttons - 4;
				}
			}
			

			if ((XIo_In32(XPAR_CHAR_LCD_BASEADDR)^buttons)& 0x2) {
				
				if(XIo_In32(XPAR_CHAR_LCD_BASEADDR) & 0x2) {
					XGpio_mSetDataReg(XPAR_LEDS_BASEADDR, 1, 0x30);
					buttons = buttons + 2;
				}
				else {
					XGpio_mSetDataReg(XPAR_LEDS_BASEADDR, 1, 0x00);
					buttons = buttons - 2;
				}
			}

			if ((XIo_In32(XPAR_CHAR_LCD_BASEADDR)^buttons)& 0x1) {
				
				if(XIo_In32(XPAR_CHAR_LCD_BASEADDR) & 0x1) {
					XGpio_mSetDataReg(XPAR_LEDS_BASEADDR, 1, 0xc0);
					buttons = buttons + 1;
				}
				else {
					XGpio_mSetDataReg(XPAR_LEDS_BASEADDR, 1, 0x00);
					buttons = buttons - 1;
				}
			}
		}

		chRead = uartScan();
		switch(chRead) {

		    case '1':
			   clearScreen();
				print("DDR Test\r\n");
				ddr_check();
				print("  Press any key to continue\r\n");
				chRead = uartRead();
				DoShowMenu();
				break;
			case '2':
			   clearScreen();
				print("Flash Test\r\n");
				flash_check();
				print("  Press any key to continue\r\n");
				chRead = uartRead();
				DoShowMenu();
				break;
			case '3':
			   clearScreen();
				print("Ethernet Test\r\n");
				DoEthTest(); //terminated by keystroke
				DoShowMenu();
				break;	
			case 'x':
					goto BIST;
				break;
	    }
	}

BIST:
   //enter silent BIST mode
	fVerbose = 0;
	print("x.p\n");
	DoTestLoop();

	return 0;
	
}



/* ------------------------------------------------------------ */
/***	DoTestLoop
**
**  Synopsis
**		DoTestLoop()
**
**  Input:
**		none
**
**  Output:
**		none
**
**  Errors:
**		none
**
**  Description:
**		This routine is the entrance to the BIST and has the 
**		main polling loop for buttons, LEDs, and received commands
**		via the RS232 port.
*/

void
DoTestLoop() {

	unsigned char ENT_pushed = 0;
	volatile int *psw_data;
	unsigned char chGet = 0;
	unsigned int buttons;
	int temp;
	int Status;

	XGpio_mSetDataDirection(XPAR_CHAR_LCD_BASEADDR, 1, 0xffffffff);   /* Set as outputs */
	buttons = XIo_In32(XPAR_CHAR_LCD_BASEADDR);

	while(1) {

		//Poll Buttons
	   if(	XIo_In32(XPAR_CHAR_LCD_BASEADDR)!=buttons) {

			if ((XIo_In32(XPAR_CHAR_LCD_BASEADDR)^buttons)& 0x1) {

				if(XIo_In32(XPAR_CHAR_LCD_BASEADDR) & 0x1) {
					print("f.1\n");
					buttons = buttons + 1;
				}
				else {
					print ("g.1\n"); 
					buttons = buttons - 1;
				}
			}

			if ((XIo_In32(XPAR_CHAR_LCD_BASEADDR)^buttons)& 0x2) {

				if(XIo_In32(XPAR_CHAR_LCD_BASEADDR) & 0x2) {
					print("f.2\n");
					buttons = buttons + 2;
				}
				else {
					print ("g.2\n"); 
					buttons = buttons - 2;
				}
			}

			if ((XIo_In32(XPAR_CHAR_LCD_BASEADDR)^buttons)& 0x4) {
				
				if(XIo_In32(XPAR_CHAR_LCD_BASEADDR) & 0x4) {
					print("f.3\n");
					buttons = buttons + 4;
				}
				else {
					print ("g.3\n"); 
					buttons = buttons - 4;
				}
			}

			if ((XIo_In32(XPAR_CHAR_LCD_BASEADDR)^buttons)& 0x8) {
				
				if(XIo_In32(XPAR_CHAR_LCD_BASEADDR) & 0x8) {
					print("f.4\n");
					buttons = buttons + 8;
				}
				else {
					print ("g.4\n"); 
					buttons = buttons - 8;
				}
			}
		}

		//LEDs
	   	WriteToGPOutput(XPAR_LEDS_BASEADDR, 8);

		chGet = uartScan();

		if ((chGet != 0) && ((chGet < 'b' || chGet > 'e') && chGet != 'x')) {
			print("z\n");
		}

		switch(chGet) {

			/* FX2 Open TEST */
		    case 'b':

				if (FDoFx2OpenTest() == 1) {
					print("b.p\n");
				}
				else {
					print("b.f\n");
				}
				break;

			/* DDR RAM TEST */
		    case 'c':
			 
				if(ddr_check()) {
					print("c.p\n");
				}
				else {
					print("c.f\n");
				}
				break;
	  
			/* FLASH TEST */            
		    case 'd':
				if(flash_check()) {
					print("d.p\n");
				}
				else {
					print("d.f\n");
				}
				break;

			case 'e':
				DoEthTest();
				print("e.p\n");
				break;
	    }
 	}
}

/* ------------------------------------------------------------ */
/***	FDoFx2OpenTest
**
**  Synopsis
**		FDoFx2OpenTest()
**
**  Input:
**		none
**
**  Output:
**		none
**
**  Errors:
**		none
**
**  Description:
**		This routine runs the FX2 open test which relies on the FX2 
**		loopback connector.
*/

int
FDoFx2OpenTest() {

	int		fRet = 1;

	Xuint32	idTestSnd = 0x00000001;
	Xuint32 idTestRcv = 0x00000003;
	Xuint32	idGet;
	int 	i;

	for (i = 0; i<16; i++) {
		XGpio_mSetDataDirection(XPAR_DAAC_DIFF_IO_BASEADDR, 1, (~idTestSnd));
		XGpio_mSetDataReg(XPAR_DAAC_DIFF_IO_BASEADDR, 1, (~idTestSnd));
		idGet = XGpio_mGetDataReg(XPAR_DAAC_DIFF_IO_BASEADDR, 1);

		if (idGet != (~idTestRcv)) {
			fRet = 0;
		}

		idTestSnd = idTestSnd << 2;
		idTestRcv = idTestRcv << 2;
	}

	return fRet;
}
