#include "ping.h"
#include "xemac.h"
#include "sleep.h"

extern char inbyte(void);
extern int fVerbose;

#define VDATA (*(unsigned*)VADDRESS)
unsigned VADDRESS;

#define MDIO_CTL_OFFSET 0x1114
#define MDIO_DATA_OFFSET 0x1118

/**
 * The largest EMAC frame in word
 */
#define XEM_MAX_FRAME_SIZE_IN_WORDS ((XEM_MAX_FRAME_SIZE / sizeof(Xuint32)) + 1)

#define ARP_LENGTH_IN_BYTE  60      /* EMAC min length is 60 byte while ARP itself is 42 byte */

#define PING_ECHO_LENGTH_IN_BYTE  60      /* EMAC min length is 60 byte while Ping Echo IP is 28 byte */

#define ICMP_LEN    8     /*ICMP is at least 8 bytes */

// vg 7/5/06
#define MEM_REG (*(unsigned*)MEM_ADDR)
unsigned MEM_ADDR;
unsigned emac_read_reg(unsigned reg_addr);
static void emac_write_reg(unsigned reg_addr, unsigned data);
//

static Xuint32 TxFrame[XEM_MAX_FRAME_SIZE_IN_WORDS];    /* word aligned */
static Xuint32 RxFrame[XEM_MAX_FRAME_SIZE_IN_WORDS];    /* word aligned */
static Xuint8 ARPPacket[ARP_LENGTH_IN_BYTE];
static Xuint8 PingEcho[XEM_MAX_FRAME_SIZE];
static Xuint8 PingEchoFrame[XEM_MAX_FRAME_SIZE];

int makePingEcho(Xuint16 id, Xuint16 seq, Xuint8* optionalPtr, Xuint32 pingLen);
int makePingEchoFrame(Xuint8* dstEmacAddr, Xuint32 srcAddr, Xuint32 dstAddr, Xuint16 ipLen, Xuint16 pingLen);
/**
 * embbedded ping host IP address
 */
Xuint8 IP_ADDR[4] = {149,199,6,18};

/**
 * For test
 */
Xuint8 JIBIN_LAPTOP_IP[4] = {198,168,0,1};

/**
 * EMAC broadcast address
 */
Xuint8 EMAC_BRDCST_ADDR[6] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

/**
 * embbedded ping host EMAC address
 */
Xuint8 EMAC_ADDR[6] = {0x02, 0x34, 0x56, 0x78, 0x9a, 0xbc};

/**
 * Read a string from stdin device
 * 
 * Number of byte read returns or -1 if fails. the trailing 0 is not counted.
 *
 */
int readStr(char* buf, int nbytes){
  int i = 0;

  for (i = 0; i < nbytes; i++) {
    *(buf + i) = inbyte();
	outbyte(*(buf + i));			// added to echo char to stdout
    if ((*(buf + i) == '\n') || (*(buf + i) == '\r')) {
      (*(buf + i)) = 0;
      break;
    }
  }
  xil_printf("\n\r");
  return (i);
}

/**
 * Set embbedded ping host IP address. The argument is like "1.2.3.4"
 *
 * Return 0 if successful
 *
 */
int setIPAddr(char* addrStr){
   Xuint8 fst, snd, trd, last;
   char* beg = addrStr;
   char* end;
   Xuint8* ipPtr = IP_ADDR;
   
   end = strchr(beg, '.');
   if (end == NULL){
      xil_printf("syntax error!\n\r");
      return -1;
   }
   --end;
   fst = (Xuint8)strtol(beg, &end, 10);   // after this operation "end" is moved to the next position
   
   beg = ++end;      // skip '.'
   end = strchr(beg, '.');
   if (end == NULL){
      xil_printf("syntax error!\n\r");
      return -1;
   }
   --end;
   snd = (Xuint8)strtol(beg, &end, 10);
   
   beg = ++end;   // skip '.'
   end = strchr(beg, '.');
   if (end == NULL){
      xil_printf("syntax error!\n\r");
      return -1;
   }
   --end;
   trd = (Xuint8)strtol(beg, &end, 10);

   beg = ++end;   // skip '.'
   last = (Xuint8)atoi(beg);

   *ipPtr++ = fst;
   *ipPtr++ = snd;
   *ipPtr++ = trd;
   *ipPtr++ = last;
   
   return 0;
}//setIPAddr

void printIP(Xuint32 ip){
   Xuint8* ptr = (Xuint8*)&ip;
   xil_printf("%d.", *ptr++);
   xil_printf("%d.", *ptr++);
   xil_printf("%d.", *ptr++);
   xil_printf("%d", *ptr);
}//printIP

void printEmac(Xuint8* ptr){
   xil_printf("%x:%x:%x:%x:%x:%x", *ptr++,*ptr++,*ptr++,*ptr++,*ptr++,*ptr);
}//printEmac

void copyWord(Xuint8* src, Xuint8* dst){
   Xuint8* sp, *dp;
   sp = src;
   dp = dst;
   *dp++ = *sp++;
   *dp++ = *sp++;
   *dp++ = *sp++;
   *dp++ = *sp++;
}//copyWord

int init(XEmac* InstancePtr){
   Xuint16 DeviceId = EMAC_DEV_ID;   
   XStatus Result;
   Xuint32 Options;                            /* device options */
   int i;
	   
   /* Init RxFrame to all 0 */
   memset((Xuint8*)RxFrame, 0, XEM_MAX_FRAME_SIZE_IN_WORDS << 2);

    /*
     * Initialize the XEmac component. The device ID chosen should be
     * configured without DMA (see the xemac_g.c file)
     */

    Result = XEmac_Initialize(InstancePtr, DeviceId);
    if (Result != XST_SUCCESS){
       return -1;
    }
    
    /*
    * Run self-test on the device, which verifies basic sanity of the
     * device and the driver.
     */
    Result = XEmac_SelfTest(InstancePtr);
    if (Result != XST_SUCCESS){
       return -1;
    }
   
    /*
     * First configure the device into poll mode.  For debug purpose, i also set 
     * Promiscuous mode so that any EMAC address frame is received.By default 
     * unicast and 
     * broadcast addressing are on.  Also, set the MAC address.
     */  

    Options = XEmac_GetOptions(InstancePtr);
    Options |= (XEM_POLLED_OPTION | XEM_PROMISC_OPTION /*added*/| XEM_FDUPLEX_OPTION);
   
    Result = XEmac_SetOptions(InstancePtr, Options);
    if (Result != XST_SUCCESS){
       return -1;
    }

    Result = XEmac_SetMacAddress(InstancePtr, EMAC_ADDR);
    if (Result != XST_SUCCESS)
    {
       return -1;
    }
    
    /*
     * Start the device, which enables the transmitter and receiver
     */
    Result = XEmac_Start(InstancePtr);
    if (Result != XST_SUCCESS){
       return -1;
    }
   
    /*
     * We clear the driver statistics so we can look at them later
     */
    XEmac_ClearStats(InstancePtr);
	
	 // for debug... print("\r\n Init done\r\n");
}//init

/**
 * Parse an IP package.
 *
 * Arguments:
 * "ip" is the target IP package to be analized; "payloadLen" is the length in 
 * byte of the IP payload; "srcAddr", "dstAddr", "protocol" and "datagram" are 
 * the IP source address, destination address, 
 * protocol and datagram of the IP package.
 *
 * Return 0 if successful
 *
 */
int parseIP(Xuint8* ip, Xuint32* srcAddr, Xuint32* dstAddr, 
            Xuint8* protocol, Xuint8** datagram, Xuint16* ipLen, Xuint16* ipDataLen){
   int i;
   int headerLen;       // length of header in 32-bit
   Xuint8* ipPtr = ip;
   
   #ifdef dbg
      xil_printf("Here are the IP package data:\n");
      for (i=0; i<64; ++i){
         xil_printf("%x ", *ipPtr++);
         if (i/8*8 == i){
            xil_printf("\n");
         }
      }
      ipPtr = ip;
   #endif
   
   headerLen = (*ipPtr) & 0x0F;  // 4th - 7th bit is header length
   #ifdef dbg
      xil_printf("IP header length is %d\n\r", headerLen);
   #endif
   
   ipPtr += 2;
   *ipLen = *((Xuint16*)(ipPtr));
   #ifdef dbg
      xil_printf("IP data gram length is %d in byte\n\r", *ipLen);
   #endif
   
   *ipDataLen = *ipLen - headerLen*4;
   
   ipPtr += 7;
   *protocol = *ipPtr;     // 10th byte: protocol
   #ifdef dbg
      xil_printf("IP protocal is %d\n\r", *protocol);
   #endif
   
   ipPtr += 3;
   #ifdef dbg
      xil_printf("Now is %dth byte deep into the IP datagram\n\r", ipPtr-ip);
      xil_printf("ipPtr=%x\n\r", ipPtr);
   #endif
   copyWord(ipPtr, srcAddr);           // 13th - 16th byte: source IP address
   ipPtr += 4;
   
   #ifdef dbg
      xil_printf("*srcAddr=%x\n\r", *srcAddr);
      xil_printf("*ipPtr=%x\n\r", *((Xuint32*)ipPtr));
   #endif
   copyWord(ipPtr, dstAddr);           // 17th - 20th byte: destination IP address
   
   // skip optional header
   ipPtr = ip;
   ipPtr += headerLen*4 ;
   
   (*datagram) = ipPtr;
   
   return 0;
}//parseIP

int isArp(Xuint8* FramePtr, Xuint8* dstEmacAddr, Xuint8* srcEmacAddr, Xuint8* srcIPAddr){
   Xuint8* ptr = (Xuint8*)FramePtr;
   ptr += 12;
   if (*((Xuint16*)ptr) != 0x0806){
      return 0;
   }

   *dstEmacAddr++ = *FramePtr++;
   *dstEmacAddr++ = *FramePtr++;
   *dstEmacAddr++ = *FramePtr++;
   *dstEmacAddr++ = *FramePtr++;
   *dstEmacAddr++ = *FramePtr++;
   *dstEmacAddr++ = *FramePtr++;
   
   *srcEmacAddr++ = *FramePtr++;
   *srcEmacAddr++ = *FramePtr++;
   *srcEmacAddr++ = *FramePtr++;
   *srcEmacAddr++ = *FramePtr++;
   *srcEmacAddr++ = *FramePtr++;
   *srcEmacAddr++ = *FramePtr++;
   
   FramePtr +=16;
   
   *srcIPAddr++ = *FramePtr++;
   *srcIPAddr++ = *FramePtr++;
   *srcIPAddr++ = *FramePtr++;
   *srcIPAddr++ = *FramePtr++;
   
   return 1;
}//isArp

int makeArp(Xuint8* dstEmacAddr, Xuint32* dstIPAddr, Xboolean arpQuery){
   Xuint8* ptr = ARPPacket;
   Xuint8* ptr2 = IP_ADDR;
   Xuint8* ptr3 = (Xuint8*)dstIPAddr;
   Xuint8* ptr4 = EMAC_ADDR;
   Xuint8* cnst_ptr = EMAC_BRDCST_ADDR;
   Xuint8* dstEmacAddrPtr = dstEmacAddr;
   
   if (arpQuery == XTRUE){
      *ptr++ = *cnst_ptr++;
      *ptr++ = *cnst_ptr++;
      *ptr++ = *cnst_ptr++;
      *ptr++ = *cnst_ptr++;
      *ptr++ = *cnst_ptr++;
      *ptr++ = *cnst_ptr++;
   }
   else{
      *ptr++ = *dstEmacAddrPtr++;
      *ptr++ = *dstEmacAddrPtr++;
      *ptr++ = *dstEmacAddrPtr++;
      *ptr++ = *dstEmacAddrPtr++;
      *ptr++ = *dstEmacAddrPtr++;
      *ptr++ = *dstEmacAddrPtr++;
   }
   
   *ptr++ = *ptr4++;
   *ptr++ = *ptr4++;
   *ptr++ = *ptr4++;
   *ptr++ = *ptr4++;
   *ptr++ = *ptr4++;
   *ptr++ = *ptr4++;
   
   *((Xuint16*)ptr)++ = 0x0806;  // ARP
   
   *((Xuint16*)ptr)++ = 1;       // Hardware type is 1 -- Ethernet
   
   *((Xuint16*)ptr)++ = 0x0800;  // Protocol type is 0x0800 -- IP

   *((Xuint16*)ptr)++ = 0x0604;  // Hardware address and protocol address sizes are 6 and 4 bytes
   
   if (arpQuery == XTRUE){
      *((Xuint16*)ptr)++ = 0x01;    // operation is ARP query
   }
   else{
      *((Xuint16*)ptr)++ = 0x02;    // operation is ARP reply
   }
   
   ptr4 = EMAC_ADDR;
   *ptr++ = *ptr4++;
   *ptr++ = *ptr4++;
   *ptr++ = *ptr4++;
   *ptr++ = *ptr4++;
   *ptr++ = *ptr4++;
   *ptr++ = *ptr4++;

   *ptr++ = *ptr2++;
   *ptr++ = *ptr2++;
   *ptr++ = *ptr2++;
   *ptr++ = *ptr2++;
   
   if (arpQuery == XTRUE){
      ptr += 6;               // skip dst emac addr for ARP query
   }
   else{
      dstEmacAddrPtr = dstEmacAddr;
      *ptr++ = *dstEmacAddrPtr++;
      *ptr++ = *dstEmacAddrPtr++;
      *ptr++ = *dstEmacAddrPtr++;
      *ptr++ = *dstEmacAddrPtr++;
      *ptr++ = *dstEmacAddrPtr++;
      *ptr++ = *dstEmacAddrPtr++;
   }

   *ptr++ = *ptr3++;
   *ptr++ = *ptr3++;
   *ptr++ = *ptr3++;
   *ptr++ = *ptr3++;
   
   return 0;
}//makeArp



/**
 * Parse the IP datagram to get ICMP fields.
 *
 * Arguments:
 * "data" is the target IP datagram; "code", "type", "id" and "seq" are the ICMP 
 * fields code, type, identification and sequence.
 *
 * Return 0 if successful
 *
 */
int parseIPDatagram(Xuint8* data, Xuint8* code, Xuint8* type, Xuint16* id, 
                    Xuint16* seq, Xuint8** optionalPtr){
   *type = *data++;
   *code = *data++;
   data++;
   data++;
   #ifdef dbg
      xil_printf("data=%x\n\r", *data);
   #endif
   *id = *((Xuint16*)data);
   data++;
   data++;
   *seq = *((Xuint16*)data);
   #ifdef dbg
      xil_printf("id = %d, seq = %d\n\r", *id, *seq);
   #endif
   data++;
   data++;
   *optionalPtr = data;
   return 0;
}//parseIPDatagram


XStatus sendArp(XEmac* InstancePtr){
   return XEmac_PollSend(InstancePtr, ARPPacket, ARP_LENGTH_IN_BYTE);
}//sendArp



int recvProcFrame(XEmac* InstancePtr){
   Xuint8 *FramePtr = NULL;
   XStatus Result = XST_NO_DATA;
   Xuint32 RecvFrameLength;
   Xuint8 srcEmacAddr[6], dstEmacAddr[6], type[2], *ui8P = NULL;
   int i, n;
	char chRead;
   Xuint32 srcIPAddr, dstIPAddr;
   Xuint8 ipProtocol;
   Xuint8* ipData = NULL;
   Xuint8 ICMPCode, ICMPType;
   Xuint16 ICMPId, ICMPSeq;
   Xuint16 ipLen;
   Xuint8* ipOptionalPtr;
   Xuint32 ipOptionalLen;
   Xuint16 pingLen;           // length in byte of ping, ICMP package
	Xuint8 chtest;

	if (fVerbose == 0) {
		makeArp(EMAC_BRDCST_ADDR, IP_ADDR, XTRUE);
		sendArp(InstancePtr);	
	}
	else {
		print("Ethernet MAC enabled.  Board can now be pinged at specified IP address.\r\n");
		print("Press any key to disable MAC and return to main menu.\r\n");
	}
		
		
   while (1){
      ui8P = NULL;
	
		if (fVerbose) {
			chRead = uartScan();
			if (chRead != 0 ) {
				return 1;
			}
		}
		
      /* this assumes MAC does not strip padding or crc */
      RecvFrameLength = XEM_MAX_FRAME_SIZE;
      Result = XEmac_PollRecv(InstancePtr, (Xuint8 *)RxFrame, &RecvFrameLength);
      if (Result == XST_SUCCESS){   
         int Index;
         FramePtr = (Xuint8 *)RxFrame;
         if (isArp(FramePtr, dstEmacAddr, srcEmacAddr, (Xuint8*)&srcIPAddr)){
							
				if (fVerbose == 0) {	//loopback success.  Return pass
					return 1;
				}
				else {	//echo received ping
				
					makeArp(srcEmacAddr, &srcIPAddr, XFALSE);
				   if (sendArp(InstancePtr) == XST_SUCCESS){
						#ifdef dbg  
							xil_printf("ARP echo sent. \n\r"); 
						#endif
					}
					else{
						#ifdef dbg  
							xil_printf("ARP echo send fails. \n\r"); 
						#endif
					}
					continue;
				}
         }

         /* get source EMAC address*/
         ui8P = dstEmacAddr;
         for (i = 0; i< 6; ++i){
            *ui8P++ = *FramePtr++;
         }

         /* get destination EMAC address*/
         ui8P = srcEmacAddr;
         for (i = 0; i< 6; ++i){
            *ui8P++ = *FramePtr++;
         }
         
         ui8P = type;
         *ui8P++ = *FramePtr++;
         *ui8P++ = *FramePtr++;
         #ifdef dbg 
            xil_printf("Source EMAC address is ");
            printEmac(srcEmacAddr);
            xil_printf("\n\r");
            xil_printf("Destination EMAC address is ");
            printEmac(dstEmacAddr);
            xil_printf("\n\r");
            xil_printf("Emac Frame Type is %x%x\n\r", type[0], type[1]);
         #endif
         if ((strncmp(dstEmacAddr, EMAC_BRDCST_ADDR, 6) != 0) && (strncmp(dstEmacAddr, EMAC_ADDR, 6) != 0)){
            continue;
         }
         #ifdef dbg 
            xil_printf("Catch this EMAC frame!\n\r");
         #endif
         if (parseIP(FramePtr, &srcIPAddr, &dstIPAddr, &ipProtocol, &ipData, &ipLen, &pingLen) != 0){
            return -1;
         }
         #ifdef dbg 
            xil_printf("Source IP address is ");
            printIP(srcIPAddr);
            xil_printf("\n\r");
            xil_printf("Destination IP address is ");
            printIP(dstIPAddr);
            xil_printf("\n\r");
         #endif
                  
         if (*((Xuint32*)IP_ADDR) != dstIPAddr){
            continue;
         }
         #ifdef dbg 
            xil_printf("Catch this IP package!\n\r");
         #endif
         
         if (ipProtocol != 1){
            #ifdef dbg 
               xil_printf("This is not an ICMP package!\n\r");
            #endif
            continue;
         }
         parseIPDatagram(ipData, &ICMPCode, &ICMPType, &ICMPId, &ICMPSeq, &ipOptionalPtr);
         if (!((ICMPCode == 0) && (ICMPType == 8))){
            #ifdef dbg 
               xil_printf("This is not a ping request!ICMPCode=%x ICMPType=%x\n\r", ICMPCode, ICMPType); 
            #endif
            
            continue;
         }
         #ifdef dbg 
            xil_printf("catch ping request!\n\r"); 
         #endif
         
         // make ping echo
         makePingEcho(ICMPId, ICMPSeq, ipOptionalPtr, pingLen);
         
         makePingEchoFrame(srcEmacAddr, dstIPAddr, srcIPAddr, ipLen, pingLen);
         
         if (XEmac_PollSend(InstancePtr, PingEchoFrame, ipLen+14) == XST_SUCCESS){
            #ifdef dbg 
               xil_printf("ping echo sent\n\r"); 
            #endif
         }
         else{
            #ifdef dbg 
               xil_printf("fail in sending ping echo\n\r"); 
            #endif
         }
      }
   }
   return 0;
}//recvProcFrame

Xuint16 checkSum(Xuint16* buffer, int size) 
{
    unsigned long cksum = 0;
    
    // Sum all the words together, adding the final byte if size is odd
    while (size > 1) {
        cksum += *buffer++;
        size -= sizeof(Xuint16);
    }
    if (size) {
        cksum += *(Xuint8*)buffer;
    }

    // Do a little shuffling
    cksum = (cksum >> 16) + (cksum & 0xffff);
    cksum += (cksum >> 16);
    
    // Return the bitwise complement of the resulting mishmash
    return (Xuint16)(~cksum);
}

/**
 * Make an ICMP package.
 * 
 * Arguments:
 * "echo" is the ping echo package to be made; "len" is the length in byte of 
 * ping echo package; "id" and "seq" are the identification and sequence of the 
 * ICMP package.
 *
 * Return 0 if successful
 *
 */
int makePingEcho(Xuint16 id, Xuint16 seq, Xuint8* optionalPtr, Xuint32 pingLen){
   Xuint8* echo = PingEcho;
   
   // type and code are 0 for ping echo
   *echo++ = 0;
   *echo++ = 0;
   
   // set checksum to 0
   *((Xuint16*)echo) = 0;
   
   echo++;
   echo++;
   
   //5th-6th byte: id
   *((Xuint16*)echo)++ = id;
   
   // 7th - 8th byte: seq
   *((Xuint16*)echo)++ = seq;
   
   memcpy(echo, optionalPtr, pingLen - ICMP_LEN);
   
   // 2nd -3rd byte: check sum
   echo = PingEcho;
   echo++;
   echo++;
   *(Xuint16*)echo = checkSum(PingEcho, pingLen);
   
   return 0;
}//makePingEcho


/**
 * Make an IP packge.
 * 
 * Arguments:
 * "srcAddr" and "dstAddr" are the IP datagram, source IP address and 
 * destination IP address.
 *
 * Return 0 if successful
 *
 */
int makePingEchoFrame(Xuint8* dstEmacAddr, Xuint32 srcAddr, Xuint32 dstAddr, Xuint16 ipLen, Xuint16 pingLen){
   Xuint8* ip = PingEchoFrame;
   Xuint16 csum, i;
   
   *ip++ = dstEmacAddr[0];
   *ip++ = dstEmacAddr[1];
   *ip++ = dstEmacAddr[2];
   *ip++ = dstEmacAddr[3];
   *ip++ = dstEmacAddr[4];
   *ip++ = dstEmacAddr[5];
   
   *ip++ = EMAC_ADDR[0];
   *ip++ = EMAC_ADDR[1];
   *ip++ = EMAC_ADDR[2];
   *ip++ = EMAC_ADDR[3];
   *ip++ = EMAC_ADDR[4];
   *ip++ = EMAC_ADDR[5];
   
   *((Xuint16*)ip) = 0x0800;     // type is IP

   ip += 2;

   *ip++ = 0x45;     // IPv4, 5 32bit wide header
   
   *ip++ = 0;          // service : 0
   
   *((Xuint16*)ip)++ = ipLen;   // length, including header
   
   *((Xuint16*)ip)++ = 1;     // identification : 1
   
   *((Xuint16*)ip)++ = 0;     // no fragment
   
   *ip++ = 3;                 // TTL is 3 seconds
   
   *ip++ = 1;                 // protocol is ICMP
   
   *((Xuint16*)ip)++ = 0;     // intialize checksum with 0
   
   //*((Xuint32*)ip)++ = srcAddr;  // source IP address
   copyWord(&srcAddr, ip);       // source IP address
   ip += 4;
   
   //*((Xuint32*)ip)++ = dstAddr;  // destination IP address
   copyWord(&dstAddr, ip);       // destination IP address
   ip += 4;
   
   csum = checkSum(ip-20, ipLen);   // set header checksum
   
   *((Xuint16*)(ip-10)) = csum;
 
   for (i =0; i < pingLen; ++i){
      *ip++ = PingEcho[i];
   }
   return 0;
   
}//makePingEchoFrame

// ----------------------------------------------------------------------------
// -------------------------------------------kl---------------------------------

int DoEthTest(){
   Xuint8 ipStr[128];
   int n;
   int i;
	int i1, i2, i3;
   XEmac Emac;                                 /* allocate a driver instance */
   XEmac *InstancePtr = &Emac;
   Xuint32 x;
	unsigned mdio_data;

   Xuint32 PhyAddress = 0x00;   
   Xuint16 *PhyDataPtr;
  
	if (fVerbose) {	//run ping test instead of loopback test.  Capture IP address
		xil_printf("Set IP address : ");
		if ((n = readStr(ipStr, 128)) == -1){
			xil_printf("Read string fails!\n\r");
			return -1;
		}
		if (setIPAddr(ipStr) != 0){
			xil_printf("IP address error!\n\r");
		}
  }
  
   if (init(InstancePtr)== -1){
		if (fVerbose) {
			xil_printf("Init fails!\n\r");
		}
		return -1;
	}

	// Read reg 2
	mdio_data = emac_read_reg(0x2); // 
	mdio_data = emac_read_reg(0x3); // 

	// *** Perform Soft Reset ***
	// Read reg 0
	mdio_data = emac_read_reg(0x0); 

	// mask bits 13 and 6 to '0' for 10Mbps
	mdio_data = (mdio_data & 0xDFBF);//1101 1111 1011 1111
	// mask bits 13 High for 100Mbps  //change to giga
	mdio_data = (mdio_data | 0x2000);
	
	// Write new value to reg 0
	emac_write_reg(0x0, mdio_data);

	// Read reg 0
	mdio_data = emac_read_reg(0x0); 

	// mask bits 15 to '1' to issue soft reset
	mdio_data = (mdio_data | 0x8000);

	// Write new value to reg 0
	emac_write_reg(0x0, mdio_data);
	
	//delay, wait for emac to reset
	if (fVerbose) {
		xil_printf("Initializing...\n\r");
	}
	sleep(3);//M00TODO:  FIX THIS!
	
   if (recvProcFrame(InstancePtr) == -1){
      return -1;
   }

   return 0;
}//main


// Following vg 7/5/06...

/////////////////////////////////////////////
//  EMAC Read PHY Reg Function             //
// reg_addr = hex address of reg to read.  //
// returns hex contents of desired register//
/////////////////////////////////////////////
unsigned emac_read_reg(unsigned reg_addr)
{
	unsigned x = 0;
	int i = 0;

	MEM_ADDR = (XPAR_ETHERNET_MAC_BASEADDR + MDIO_CTL_OFFSET);

//  The MAC mdio control register is as follows:
//            0| 1| 2-6   | 7-11  |12|13
//           SB|RW|PHYADDR|REGADDR|IE|RE
//           "0  1  00000   00000   1  0"
//	         "01PP PPPR RRRR 10" 00 0000 0000 0000 0000;
//	Use mask value of "C0080000" plus desired shifted PHY and REG address;
//  Assume a phy address of "0" for now.
//  SB bit will be cleared by MAC when read is done.


    // Shift by 20 to position correctly in MAC control reg (see bits above)
	x = 0xC0080000 + (reg_addr<<20);

	MEM_REG = x;              // write to MAC mdio control register
                              // to set up Read from desired Phy Reg

    // Small delay for mdio transaction.  Should read status bit here instead
	for (i=0; i<100000; i++);

	MEM_ADDR = (XPAR_ETHERNET_MAC_BASEADDR + MDIO_DATA_OFFSET);
	x = MEM_REG; // read from MAC mdio data register


	return x;
}
/////////////////////////////////////////////

/////////////////////////////////////////////
//  EMAC Write PHY Reg Function            //
// reg_addr = hex address of reg to read.  //
// data = hex data to write                //
/////////////////////////////////////////////
static void emac_write_reg(unsigned reg_addr, unsigned data)
{
	unsigned x = 0;
	int i = 0;


//  The MAC mdio control register is as follows:
//            0| 1| 2-6   | 7-11  |12|13
//           SB|RW|PHYADDR|REGADDR|IE|RE
//           "0  0  00000   00000   1  0"
//	         "00PP PPPR RRRR 10" 00 0000 0000 0000 0000;
//	Use mask value of "80080000" plus desired shifted PHY and REG address;
//  Assume a phy address of "0" for now.
//  SB bit will be cleared by MAC when read is done.


	// load new data into MAC mdio data register	  
	MEM_ADDR = (XPAR_ETHERNET_MAC_BASEADDR + MDIO_DATA_OFFSET);
	MEM_REG = data; // put data in the MAC mdio data register

	// Initiate the mdio write
	MEM_ADDR = (XPAR_ETHERNET_MAC_BASEADDR + MDIO_CTL_OFFSET);
	MEM_REG  = 0x80080000 + (reg_addr << 20); // mdio write register

    // Small delay for mdio transaction.  Should RD status bit here instead
	for (i=0; i<400000; i++);

}
/////////////////////////////////////////////
