//#include <xi2c_l.h>
#include "sdram_detect.h"

#define I2C_GPO_OFFSET 0x124
#define DDR_SLAVE_ADDR 0x54

static Xuint32 I2C_BaseAddress;

int readReg(Xuint8 reg_addr);


void i2cInit(Xuint32 base_address) {
  I2C_BaseAddress = base_address;
}


int sdramDetect() {
  return sdramDetectX(1);
}

int sdramDetectX(int verbose) {
  int mem_type, row_count, col_count, rank_count;

  /** Check the memory type **/
  mem_type = readReg(2);
  if( mem_type == -1 ) {
    if(verbose)
      print("No DIMM module detected.\r\n");
    return 0;
  }

  if( mem_type != 7 ) {
    if(verbose) {
      print("DIMM module not supported:\r\n");
      xil_printf("  Invalid type: %d.\r\n", mem_type);
    }
    return 0;
  }

  /** Check to see if the module is supported by this BIST **/
  row_count = readReg(3);
  col_count = readReg(4);
  rank_count = readReg(5);

  if( row_count != 13 ) {
    if(verbose) {
      print("DIMM module not supported:\r\n");
      xil_printf("  Invalid row address count: %d.\r\n", row_count);
      print("  Required row address count: 13.\r\n");
    }
    return 0;
  }

  if( col_count != 10 ) {
    if(verbose) {
      print("DIMM module not supported:\r\n");
      xil_printf("  Invalid column address count: %d.\r\n", col_count);
      print("  Required column address count: 10.\r\n");
    }
    return 0;
  }

  if( rank_count > 2 || rank_count < 1 ) {
    if(verbose) {
      print("DIMM module not supported:\r\n");
      xil_printf("  Invalid rank count: %d.\r\n", rank_count);
      print("  Only 1 or 2 ranks supported.\r\n");
    }
    return 0;
  }

  return rank_count;
  
}


int readReg(Xuint8 reg_addr) {
  Xuint8 recv_cnt;
  Xuint8 recv_data[1];

  XI2c_mWriteReg(I2C_BaseAddress, I2C_GPO_OFFSET, 1);
  XI2c_mWriteReg(I2C_BaseAddress, I2C_GPO_OFFSET, 0);
  recv_cnt = XI2c_RSRecv(I2C_BaseAddress, DDR_SLAVE_ADDR, reg_addr, 
			 recv_data, 1);
  if(recv_cnt != 1) {
    return -1;
  }

  return recv_data[0];

}
