/*******************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY forUSE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE forOBTAINING ANY RIGHTS YOU MAY REQUIRE
*     forYOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     forA PARTICULAR PURPOSE.
*     
*     (c) Copyright 2004 Xilinx, Inc.
*     All rights reserved.
*
******************************************************************************/

#include <stdio.h>
#include "sleep.h"
#include "xgpio_l.h"
#include "xparameters.h"

#define LCD_BASEADDR XPAR_CHAR_LCD_BASEADDR
#define INIT_DELAY 5000 //usec delay timer during initialization, important to change if clock speed changes
#define INST_DELAY 5000 //usec delay timer between instructions
#define DATA_DELAY 250  //usec delay timer between data

/*
#------------------------------------------------------------------------------
# IO Pad Location Constraints / Properties for Character LCD GPIO
#------------------------------------------------------------------------------

NET  Char_LCD_pin<6>           LOC = M18  |  PULLDOWN;  # LCD_E
NET  Char_LCD_pin<5>           LOC = L18  |  PULLDOWN;  # LCD_RS
NET  Char_LCD_pin<4>           LOC = L17  |  PULLDOWN;  # LCD_RW
NET  Char_LCD_pin<3>           LOC = N9   |  PULLDOWN;  # LCD_DB7
NET  Char_LCD_pin<2>           LOC = M9   |  PULLDOWN;  # LCD_DB6
NET  Char_LCD_pin<1>           LOC = R9   |  PULLDOWN;  # LCD_DB5
NET  Char_LCD_pin<0>           LOC = U9   |  PULLDOWN;  # LCD_DB4
*/

//==============================================================================
//
//                                                              INTERNAL FUNCTIONS
//
//==============================================================================



void XromInitInst(void)
{
   XGpio_mSetDataReg(LCD_BASEADDR, 1, 0x00000003);
   usleep(1);
   XGpio_mSetDataReg(LCD_BASEADDR, 1, 0x00000043); //set enable and data
   usleep(1);
   XGpio_mSetDataReg(LCD_BASEADDR, 1, 0x00000003);
   usleep(INIT_DELAY);
}



void XromWriteInst(unsigned long inst1, unsigned long inst2)
{
unsigned long printinst;

   printinst = 0x00000040 | inst1;
        
   XGpio_mSetDataReg(LCD_BASEADDR, 1, inst1);     //write data
   usleep(1);
   XGpio_mSetDataReg(LCD_BASEADDR, 1, printinst); //set enable
   usleep(1);
   XGpio_mSetDataReg(LCD_BASEADDR, 1, inst1);     //turn off enable
   usleep(1);
        
   printinst = 0x00000040 | inst2;

   XGpio_mSetDataReg(LCD_BASEADDR, 1, printinst); //set enable and data
   usleep(1);
   XGpio_mSetDataReg(LCD_BASEADDR, 1, inst2);     //turn off enable

   usleep(INST_DELAY);     
}



void XromWriteData(unsigned long data1, unsigned long data2)
{
unsigned long rs_data, enable_rs_data;

   rs_data = 0x00000020 | data1;                       //sets rs, data1
   enable_rs_data = 0x00000060 | data1;

   XGpio_mSetDataReg(LCD_BASEADDR, 1, rs_data);        //write data, rs
   usleep(1);
   XGpio_mSetDataReg(LCD_BASEADDR, 1, enable_rs_data); //set enable, keep data, rs
   usleep(1);
   XGpio_mSetDataReg(LCD_BASEADDR, 1, rs_data);        //turn off enable
   usleep(1);
        
   rs_data = 0x00000020 | data2;                       //sets rs, data2
   enable_rs_data = 0x00000060 | data2;                //sets rs, data2

   XGpio_mSetDataReg(LCD_BASEADDR, 1, enable_rs_data); //set enable, rs, data
   usleep(1);
   XGpio_mSetDataReg(LCD_BASEADDR, 1, rs_data);        //turn off enable
        
   usleep(DATA_DELAY);
}



//==================================================================================
//
//                                                              EXTERNAL FUNCTIONS
//
//==================================================================================

void XromMoveCursorHome()
{
   XromWriteInst(0x00000000, 0x00000002);
}

void XromMoveCursorLeft()
{
   XromWriteInst(0x00000001, 0x00000000);
}

void XromMoveCursorRight()
{
   XromWriteInst(0x00000001, 0x00000004);
}

void XromLCDOn()
{
   XromWriteInst(0x00000000, 0x0000000E);
}

void XromLCDOff()
{
   XromWriteInst(0x00000000, 0x00000008);
}

void XromLCDClear()
{
   XromWriteInst(0x00000000, 0x00000001);
}



void XromLCDInit()
   {
   XGpio_mSetDataDirection(LCD_BASEADDR, 1, 0x00000000);   // Sets CHAR LCD Reg to Write Mode
   XGpio_mSetDataReg(LCD_BASEADDR, 1, 0x00000000);         // Zeroes CHAR LCD Reg

   //LCD INIT
   usleep(2*15000);                         //After VCC>4.5V Wait 15ms to Init Char LCD
   XromInitInst();
   usleep(2*4100);                          //Wait 4.1ms
   XromInitInst();
   usleep(2*100);                           //Wait 100us
   XromInitInst();
   XromInitInst();

   XromWriteInst(0x00000002, 0x00000002);   //Function Set
   XromWriteInst(0x00000000, 0x00000008);   //Display Off
   XromWriteInst(0x00000000, 0x00000001);   //Display Clear
   XromWriteInst(0x00000000, 0x00000006);   //Entry Mode Set
   XromWriteInst(0x00000000, 0x0000000E);   //Display On
}



void XromLCDSetLine(int line){ //line1 = 1, line2 = 2
        
        int i;

        if((line - 1)) {
                XromMoveCursorHome();
                for(i=0; i<40; i++)
                        XromMoveCursorRight();
        }
        else
                XromMoveCursorHome();
                
}



void XromLCDPrintChar(char c)
{
   XromWriteData(((c >> 4) & 0x0000000F), (c & 0x0000000F));
}



void XromLCDPrintString(char * line)
{
int i=0;
   while(line[i])
      {
      XromLCDPrintChar(line[i]);
      i++;
      }

   return;
}



void XromLCDPrint2Strings(char * line1, char * line2)
{
int i=0;

   XromLCDSetLine(1);
   for(i=0; i<16; i++)
      {
      if (line1[i])
         XromLCDPrintChar(line1[i]);
      else
	     break;
      }       

   XromLCDSetLine(2);
   for(i=0; i<16; i++)
      {
      if(line2[i])
         XromLCDPrintChar(line2[i]);
      else
	     break;
      }       

   return;
}
