--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    09:53:27 02/03/06
-- Design Name:    
-- Module Name:    VgaCcolorBarGen - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity VgaCcolorBarGen is

generic (PLD: integer:=800; --Pixel/Line Divider
         LFD: integer:=521); --Line/Frame Divider

  Port (Hcnt: in integer range 0 to PLD-1; -- horizontal counter
        Vcnt: in integer range 0 to LFD-1; -- verical counter
        outRed  : out std_logic_vector(7 downto 0);
        outGreen : out std_logic_vector(7 downto 0);
        outBlue  : out std_logic_vector(7 downto 0));
end VgaCcolorBarGen;

architecture Behavioral of VgaCcolorBarGen is

  constant PAL:integer:=640;  --Pixels/Active Line (pixels)
  constant LAF:integer:=480;  --Lines/Active Frame (lines)

  signal cntBand : integer range 0 to 39; -- verical counter
  signal intRed : std_logic_vector(7 downto 0);
  signal intGreen: std_logic_vector(7 downto 0);
  signal intBlue : std_logic_vector(7 downto 0);

begin

--vecHcnt <= conv_std_logic(Hcnt,10);
  cntBand <= Hcnt/16;  -- band width = 16 pixels => 40 bands

  intRed <= "00000000" when cntBand = 0 or cntBand = 30 else
             "00000001" when cntBand = 1 or cntBand = 31 else
             "00000010" when cntBand = 2 or cntBand = 32 else
             "00000100" when cntBand = 3 or cntBand = 33 else
             "00001000" when cntBand = 4 or cntBand = 34 else
             "00010000" when cntBand = 5 or cntBand = 35 else
             "00100000" when cntBand = 6 or cntBand = 36 else
             "01000000" when cntBand = 7 or cntBand = 37 else
             "10000000" when cntBand = 8 or cntBand = 38 else
             "11111111" when cntBand = 9 or cntBand = 39 else
             "00000000";

  intGreen <="00000000" when cntBand = 10 or cntBand = 30 else
             "00000001" when cntBand = 11 or cntBand = 31 else
             "00000010" when cntBand = 12 or cntBand = 32 else
             "00000100" when cntBand = 13 or cntBand = 33 else
             "00001000" when cntBand = 14 or cntBand = 34 else
             "00010000" when cntBand = 15 or cntBand = 35 else
             "00100000" when cntBand = 16 or cntBand = 36 else
             "01000000" when cntBand = 17 or cntBand = 37 else
             "10000000" when cntBand = 18 or cntBand = 38 else
             "11111111" when cntBand = 19 or cntBand = 39 else
             "00000000";
  intBlue <= "00000000" when cntBand = 20 or cntBand = 30 else
             "00000001" when cntBand = 21 or cntBand = 31 else
             "00000010" when cntBand = 22 or cntBand = 32 else
             "00000100" when cntBand = 23 or cntBand = 33 else
             "00001000" when cntBand = 24 or cntBand = 34 else
             "00010000" when cntBand = 25 or cntBand = 35 else
             "00100000" when cntBand = 26 or cntBand = 36 else
             "01000000" when cntBand = 27 or cntBand = 37 else
             "10000000" when cntBand = 28 or cntBand = 38 else
             "11111111" when cntBand = 29 or cntBand = 39 else
             "00000000";

  outRed <= intRed when Hcnt < PAL and Vcnt < LAF else
             "00000000";
  outGreen <=intGreen when Hcnt < PAL and Vcnt < LAF else
             "00000000";
  outBlue <= intBlue when Hcnt < PAL and Vcnt < LAF else
             "00000000";


end Behavioral;
