--------------------------------------------------------------------------------
-- Company: Digilent Inc.
-- Engineer: Mircea Dabacan
--
-- Create Date:    13:43:19 02/03/06
-- Design Name:    VgaTestDigilentCore
-- Module Name:    VgaTest - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity VgaTest is

generic (PLD: integer:=800; --Pixel/Line Divider
         LFD: integer:=521); --Line/Frame Divider

    Port ( clk: in std_logic;
           HS : out std_logic;
           VS : out std_logic;
           clkVgaDAc: out std_logic;
           syncVga  : out std_logic;
           outBlank : out std_logic;
           outRed   : out std_logic_vector(7 downto 0);
           outGreen : out std_logic_vector(7 downto 0);
           outBlue  : out std_logic_vector(7 downto 0));
end VgaTest;

architecture Behavioral of VgaTest is

	COMPONENT vgaccolorbargen
	PORT(
		Hcnt : integer range 0 to PLD-1;  -- horizontal counter
		Vcnt : integer range 0 to LFD-1;  -- verical counter    
		outRed  : OUT std_logic_vector(7 downto 0);
		outGreen: OUT std_logic_vector(7 downto 0);
		outBlue : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;

	COMPONENT synchro
	PORT(
		ck100MHz : IN std_logic;    
		Hcnt : integer range 0 to PLD-1;  -- horizontal counter
		Vcnt : integer range 0 to LFD-1;  -- verical counter
		ck25MHzOut : OUT std_logic;
		HS : OUT std_logic;
		VS : OUT std_logic;
		outBlank : OUT std_logic
		);
	END COMPONENT;

  signal Hcnt: integer range 0 to PLD-1;  -- horizontal counter
  signal Vcnt: integer range 0 to LFD-1;  -- verical counter

begin

	Inst_vgaccolorbargen: vgaccolorbargen PORT MAP(
		Hcnt => Hcnt,
		Vcnt => Vcnt,
		outRed => outRed,
		outGreen => outGreen,
		outBlue => outBlue
	);

	Inst_synchro: synchro PORT MAP(
		ck100MHz => clk,
		ck25MHzOut => clkVgaDac,
		HS => HS,
		VS => VS,
		outBlank => outBlank,
		Hcnt => Hcnt,
		Vcnt => Vcnt
	);

  syncVga <= '0';

end Behavioral;
